/*
 *	readconf.h
 *
 *	Copyright (C) 2004-2005 Bartomiej Korupczynski <bartek@klolik.org>
 *
 *	This program is free software; you can redistribute it and/or 
 *	modify it under the terms of the GNU General Public License 
 *	as published by the Free Software Foundation; either 
 *	version 2 of the License, or (at your option) any later 
 *	version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _CONFFILE_H_
#define _CONFFILE_H_

#include <sys/types.h>

#ifndef _CONFFILE_C_
#define EXTERN	extern
#else
#define EXTERN
#endif


/*
 * 	configuration dump macros
*/

#define CONF_S(a)	printf("%-30s %s\n", #a, a);
#define CONF_S2(a)	printf("  %-30s: %s\n", #a, a);
#define CONF_SS(a, b)	printf("%-30s %s\n", a, b);
#define CONF_SS2(a, b)	printf("  %-30s: %s\n", a, b);
#define CONF_D(a)	printf("%-30s %d\n", #a, a);
#define CONF_D2(a)	printf("  %-30s: %d\n", #a, a);
// signed int
#define CONF_II(a, b)	printf("%-30s %d\n", a, b);
// unsigned int
#define CONF_UU(a, b)	printf("%-30s %u\n", a, b);
// unsigned int octal
#define CONF_OO(a, b)	printf("%-30s 0%o\n", a, b);
// unsigned int hex
#define CONF_HH(a, b)	printf("%-30s 0x%x\n", a, b);
// double
#define CONF_DO(a, b)	printf("%-30s %f\n", a, b);


/*
 * 	configuration flags
*/
#define CONF_FLAG_NONE		0x00

// UINT variables dump flags
#define CONF_FLAG_DUMP_MASK	0x0f
#define CONF_FLAG_HEX		0x01
#define CONF_FLAG_OCT		0x02

// values validation flags
#define CONF_FLAG_VALID_MASK	0xf0
#define CONF_FLAG_MINUS_1	0x10
#define CONF_FLAG_EMPTY		0x20


typedef enum { CONFIG_END = 0,
	CONFIG_BOOL, CONFIG_INT, CONFIG_PORT,
	CONFIG_UINT,
	CONFIG_STR, CONFIG_IP, 
	CONFIG_DOUBLE
} option_type;




typedef union {
	int i;
	uint u;
	char *s;
	double d;
} config_def_val;

typedef union {
	int *i;
	uint *u;
	char *s;
	double *d;
} config_val;

typedef union {
	int uint_flags;
	int path_flags;
	int file_flags;
} config_flags;

struct config_option {
	option_type type;
	char *name;
	void *val;
	size_t size;
	int flags;
//	config_flags type_flags;
	config_def_val def;
};



EXTERN int read_config(struct config_option config_options[], char *filename);
EXTERN int config_set_defaults(struct config_option config_options[]);
EXTERN void dump_config(struct config_option config_options[], char *ignore_prefix);


#undef EXTERN

#endif


